import pkg_resources
from fastapi import FastAPI
import uvicorn
from fastapi.responses import FileResponse
from starlette.responses import HTMLResponse

from fastmodel.backend.domain.dto.req_classes import SdInpaintReqDto
from fastmodel.backend.util.str_utils import safe_join
from fastmodel.backend.inference.inference import inference_sd_inpaint

CSS_PATH_LIB = pkg_resources.resource_filename(__name__, "fastmodel/frontend/edit-anything/assets/css")
IMG_PATH_LIB = pkg_resources.resource_filename(__name__, "fastmodel/frontend/edit-anything/assets/img")
JS_PATH_LIB = pkg_resources.resource_filename(__name__, "fastmodel/frontend/edit-anything/js")

app = FastAPI()


@app.get("/", response_class=HTMLResponse)
async def server():
    return FileResponse("fastmodel/frontend/edit-anything/index.html")


@app.get("/assets/css/{path:path}")
async def build_resource(path: str):
    build_file = safe_join(CSS_PATH_LIB, path)
    return FileResponse(build_file)


@app.get("/js/{path:path}")
async def build_resource(path: str):
    js_file = safe_join(JS_PATH_LIB, path)
    return FileResponse(js_file)


@app.post("/gw/edit-anything/api/v1/bff/sd/inpaint")
async def sd_inpaint(body: SdInpaintReqDto):
    image = body.image
    mask = body.mask
    prompt = body.prompt

    # todo sd in-paint inference
    inpaint_in_base64 = inference_sd_inpaint(image, mask, prompt)
    # todo base64 to image

    return {
        "msg": "ok",
        "msgCode": "10000",
        "success": True,
        "total": 1,
        "data": {
            "imgBase64": inpaint_in_base64,
            "imgUrl": 'https://static.openxlab.org.cn/public/image/aigc_inpaint/7c7288ca-eb10-11ed-86bd-f6d908acfd33.png'
        }
    }


@app.get("/favicon.ico")
async def favicon():
    ico_file = safe_join(IMG_PATH_LIB, 'favicon.ico')
    return FileResponse(ico_file)


if __name__ == "__main__":
    uvicorn.run(app, host="127.0.0.1", port=9911)
